/*

Greetings everyone! Another small supplemental library from Bravo1! whoo!

NOTE: This library is meant to be used with the get_ procs library!

This library adds the functionality of field of view to mobs, with some simple (yet lengthy) procs.



Variables:
	atom
		fov=90 this is the default field of view, and all atoms have it in case you're not looking to use this just for mobs.

Procs:
	atom
		inFOV(atom/a) - returns true if the given atom (a) is within the fov of src. nfov can be used in place of src's fov.

		FOVlist(dist,nfov) - returns a list of all atoms at the given distance (dist) in tileswithin the FOV of the atom calling it.
		nfov is a number that can be used in place of the FOV of the atom calling the proc.

	mob
		inFOV_mobs(mob/a) - returns true if the given mon (a) is within the fov of src. Much more accurate than inFOV() but only works for mobs.
		Once again, nfov can be used in place of src's fov.

		FOVlist_mobs(dist,nfov) - the same as FOV() however it has a few key differences. dist is done in pixels instead of tiles and is then converted so that a circular range (instead of BYONDs square range) can
		be used for distance. This also uses bound_dist to find the distance to the mobs, and get_angle_step to procure a more accurate angle between src and the mobs it's checking.
		This will return a list of only the mobs in the given FOV and can only be called by mobs as a result. Once again, nfov can be used instead of the src mob's regular fov.


*/


atom
	var/fov=90
	proc

		inFOV(atom/a,nfov)
			if(!nfov) nfov=fov
			var/dfov=nfov/2
			var/angle=get_angle_dir(src)
			var/a_angle
			if(ismob(src)&&ismob(a)) a_angle=get_angle_step(src,a) //if both src and a are mobs, use the more accurate proc.
			else a_angle=get_angle(src,a)
			var/switchL=0
			var/switchR=0
			var/edgeL=angle-dfov
			var/edgeR=angle+dfov
			if(edgeL<0) switchL=1
			if(edgeR>360) switchR=1
			if(!switchL&&!switchR) if(between(a_angle,edgeL,edgeR)) return 1
			else
				if(switchL) edgeL+=360
				if(switchR) edgeR-=360
				if(!between(a_angle,edgeR,edgeL,1)) return 1





		FOVlist(dist,nfov)
			if(!dist) dist=world.view
			var/list/inview=new
			for(var/atom/a in view(dist,src)-src)
				if(inFOV(a,nfov)) inview.Add(a)
			return inview

mob
	proc

		FOVlist_mobs(dist,nfov)
			if(!dist) dist=world.view*world.icon_size
			var/vdist=(dist+world.icon_size)/world.icon_size
			var/list/inview=new
			for(var/mob/a in view(vdist,src)-src)
				if(bounds_dist(src,a)>dist) continue
				if(inFOV(a,nfov)) inview.Add(a)
			return inview






